<?php

/*
 * Project: MinecraftLogFileReader
 * File name: online.php
 * Description: Reads the server logfile of Minecraft to provide useful information
 *
 * @author Michael Csida, Copyright (C) 2011.
 * @version v 0.2.1 2011/06/18
 * @see The GNU Public License (GPL)
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*-----------------------------------------------------------------------
--- VARIABLES YOU CAN / NEED TO CHANGE ----------------------------------
-----------------------------------------------------------------------*/

$whitelist = 'white-list.txt'; 			// Name of your server whitelist file
$logfile = 'server.log'; 				// Name of your server logfile
$path_to_logs = '/home/minecraft/'; 	// Absolute or relative path to the directory where the 2 files are located

$display_minute_after = 60; 			// After 60 seconds, minutes will be displayed instead of seconds
$display_hour_after = 7200; 			// After 120 minutes (7200 seconds), hours will be displayd instead of minutes
$display_day_after = 172800; 			// After 48 hours (172800 seconds), days will be displayed instead of hours

/*-----------------------------------------------------------------------
--- CODE ----------------------------------------------------------------
-----------------------------------------------------------------------*/

// Fill variables
$file_logfile = $path_to_logs.$logfile;
$file_whitelist = $path_to_logs.$whitelist;

$array_logfile = file($file_logfile);
$array_whitelist = file($file_whitelist);
sort($array_whitelist);
$count_user = count($array_whitelist);

$current_time = time();

// Array for username, current userstatus and last change of online/offline status
for ($i = 0; $i < $count_user; $i++) {
	$array_users[$i]['name'] = rtrim(ltrim($array_whitelist[$i]));
	$array_users[$i]['status'] = 'UNCLEAR';
	$array_users[$i]['lastchange'] = 0;
	$array_users[$i]['totaltime'] = 0;
}

// Temporary LogFile Array with "[INFO]" rows only
$h = 0;
for ($i = 0; $i < count($array_logfile); $i++) {
	if (preg_match('#[INFO]#', $array_logfile[$i])) {
		$array_logfile_temp[$h] = $array_logfile[$i];
		$h++;
	}
}

// Temporary LogFile Array with Online/Offline - Info rows only
$h = 0;
for ($i = 0; $i < count($array_logfile_temp); $i++) {
	if (preg_match('#lost connection: disconnect.#', $array_logfile_temp[$i]) || preg_match('#logged in with entity id#', $array_logfile_temp[$i])) {
		$array_logfile_temp_onoff[$h] = $array_logfile_temp[$i];
		$h++;
	}
}

// Script: Finding out Online/Offline - Status and set timestamp of last change
$users_checked = 0;
$lines_to_check = count($array_logfile_temp_onoff) - 1;

while (($users_checked < $count_user) && ($lines_to_check >= 0)) {
	$array_temp1 = explode(" [INFO] ", $array_logfile_temp_onoff[$lines_to_check]);
	$array_temp2 = explode(" ", $array_temp1[1], 2);
	for ($i = 0; $i < $count_user; $i++) {
		if ((strtoupper($array_users[$i]['name']) == strtoupper($array_temp2[0])) && $array_users[$i]['status'] == 'UNCLEAR') {
			if (preg_match('#lost connection: disconnect.#', $array_temp2[1])) $array_users[$i]['status'] = 'OFFLINE';
			if (preg_match('#logged in with entity id#', $array_temp2[1])) $array_users[$i]['status'] = 'ONLINE';
			$array_temp3 = explode(" ", $array_temp1[0]);
			$array_temp4 = explode("-", $array_temp3[0]);
			$array_temp5 = explode(":", $array_temp3[1]);
			$array_users[$i]['lastchange'] = mktime($array_temp5[0],$array_temp5[1],$array_temp5[2],$array_temp4[1],$array_temp4[2],$array_temp4[0]);
			$array_users[$i]['name'] = $array_temp2[0];
			$users_checked++;
		}
	}
	$lines_to_check--;
}

// Script: Finding out Total Play Time
$array_logline_timestamp = array();
$array_logline_user = array();
$array_logline_status = array();

for ($i = 0; $i < $count_user; $i++) { // Check one user after another
	$f = 0;
	unset($array_logfile_temp_user);
	for ($g = 0; $g < count($array_logfile_temp_onoff); $g++) { // Create array for online/offline information for only this user
		$array_temp6 = explode(" [INFO] ", $array_logfile_temp_onoff[$g]);
		$array_temp7 = explode(" ", $array_temp6[1], 2);
		if (strtoupper($array_users[$i]['name']) == strtoupper($array_temp7[0])) {
			$array_logfile_temp_user[$f] = $array_logfile_temp_onoff[$g];
			$f++;
		}
	}
	for ($e = 0; $e < count($array_logfile_temp_user); $e++) { // Store all the loglines of the user-array to other arrays once
		$array_logline_temp1 = explode(" [INFO] ", $array_logfile_temp_user[$e]);
		$array_logline_temp2 = explode(" ", $array_logline_temp1[0]);
		$array_logline_temp3 = explode("-", $array_logline_temp2[0]);
		$array_logline_temp4 = explode(":", $array_logline_temp2[1]);
		$array_logline_temp5 = explode(" ", $array_logline_temp1[1], 2);

		$array_logline_timestamp[$e] = mktime($array_logline_temp4[0],$array_logline_temp4[1],$array_logline_temp4[2],$array_logline_temp3[1],$array_logline_temp3[2],$array_logline_temp3[0]);
		if (preg_match('#logged in with entity id#', $array_logline_temp5[1])) {
			$array_logline_status[$e] = 'ONLINE';
		}
		if (preg_match('#lost connection: disconnect.#', $array_logline_temp5[1])) {
			$array_logline_status[$e] = 'OFFLINE';
		}
		$array_logline_user[$e] = $array_logline_temp5[0];
	}
	for ($h = 0; $h < count($array_logfile_temp_user); $h++) { // Calculating Total Played Time
		if ($h == 0) { // First entry of the user logfile
			$temp_current_timestamp = $array_logline_timestamp[$h];
		} else if (($h > 0) && ($h < count($array_logfile_temp_user)-1)){
			if (($array_logline_status[$h] == 'ONLINE') && ($array_logline_status[$h-1] == 'OFFLINE')) {
				$temp_current_timestamp = $array_logline_timestamp[$h];
			}
			if (($array_logline_status[$h] == 'OFFLINE') && ($array_logline_status[$h-1] == 'ONLINE')) {
				$diff = $array_logline_timestamp[$h] - $temp_current_timestamp;
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
		} else if ($h == count($array_logfile_temp_user)-1) { // Last entry of the user logfile
			if ($array_logline_status[$h] == 'ONLINE') {
				$diff = $current_time - $array_logline_timestamp[$h];
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
			if ($array_logline_status[$h] == 'OFFLINE') {
				$diff = $array_logline_timestamp[$h] - $temp_current_timestamp;
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
		}
	}
}

// Sort user array (online first)
$h=0;
for ($i = 0; $i < $count_user; $i++) {
	if ($array_users[$i]['status'] == 'ONLINE') {
		$array_users_sorted[$h]['name'] = $array_users[$i]['name'];
		$array_users_sorted[$h]['status'] = $array_users[$i]['status'];
		$array_users_sorted[$h]['lastchange'] = $array_users[$i]['lastchange'];
		$array_users_sorted[$h]['totaltime'] = $array_users[$i]['totaltime'];
		$h++;
	}
}
for ($i = 0; $i < $count_user; $i++) {
	if ($array_users[$i]['status'] != 'ONLINE') {
		$array_users_sorted[$h]['name'] = $array_users[$i]['name'];
		$array_users_sorted[$h]['status'] = $array_users[$i]['status'];
		$array_users_sorted[$h]['lastchange'] = $array_users[$i]['lastchange'];
		$array_users_sorted[$h]['totaltime'] = $array_users[$i]['totaltime'];
		$h++;
	}
}

?>

<!-----------------------------------------------------------------------
--- OUTPUT --------------------------------------------------------------
------------------------------------------------------------------------>

<body background="pictures/background.png">

<head>
	<link rel=stylesheet type="text/css" href="style.css">
</head>

<br><br>

<table width=600 align=center cellpadding=3 cellspacing=3 border=0>
	<tr style="background: url(pictures/table_background_01.png)">
		<td width=25>&nbsp;</td>
		<td width=125><center><span class="vbn12white">Name</span></center></td>
		<td width=225><center><span class="vbn12white">Total Play Time</span></center></td>
		<td width=225><center><span class="vbn12white">Status</span></center></td>
	</tr>
	<?php
	for ($i = 0; $i < $count_user; $i++) {
		echo "\n<tr style=\"background: url(pictures/";
		if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "table_background_02.png";
		else echo "table_background_03.png";
		echo ")\">\n<td><center><img src=\"pictures/";
		if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "online";
		else if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo "offline";
		else echo "unclear";
		echo ".jpg\"></center></td>\n";
		$temp_totaltime_hours = sprintf("%01.1f", round($array_users_sorted[$i]['totaltime']/60/60, 1));
		$temp_totaltime_days = sprintf("%01.2f", round($array_users_sorted[$i]['totaltime']/60/60/24, 2));
		echo "<td><center><span class=\"vnn10white\">".$array_users_sorted[$i]['name']."</span></center></td>\n";
		echo "<td><center><span class=\"vnn10white\">".$temp_totaltime_hours." hours (".$temp_totaltime_days." days)</center></td>";
		echo "<td><span class=\"vnn10white\">";
		if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "Online for ";
		else if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo "Last online: ";
		else echo "Unclear";
		$diff = $current_time - $array_users_sorted[$i]['lastchange'];
		if ($array_users_sorted[$i]['lastchange'] == 0) {
			echo "";
		} else if ($diff >= $display_day_after) {
			$temp = floor($diff/86400);
			echo $temp." day"; if ($temp > 1) echo "s";
		} else if ($diff >= $display_hour_after) {
			$temp = floor($diff/3600);
			echo $temp." hour"; if ($temp > 1) echo "s";
		} else if ($diff >= $display_minute_after) {
			$temp = floor($diff/60);
			echo $temp." minute"; if ($temp > 1) echo "s";
		} else {
			echo $diff." second"; if ($diff > 1) echo "s";
		}
		if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo " ago";		
		echo "</span></td>\n";
		echo "</tr>\n";
	}
	?>
</table>

</body>